"""hekaiyue 何恺悦 2024-01-26"""
from datetime import datetime
import cv2
import numpy

def in_period(periods=[]):
    # [["06:00", "08:00"], ...] --> [[<datetime>, <datetime>], ...]
    periods = [list(map(lambda x: datetime.strptime(x, "%H:%M").time(), period)) for period in periods] if len(periods) > 0 else True
    # 如果检测时间为空则默认任何时间都在检测时间内
    if periods is True: return True
    # 比对当前时间在哪个区间内，返回所在的第一个时间区间
    now = datetime.now().time()
    for index, (period_start, period_end) in enumerate(periods):
        if period_start <= now <= period_end:
            return index
    return False

def in_region(point, regions):
    # [x, y] [[[x0, y0], [x1, y1], [x2, y2], ...], ...]
    for index, region in enumerate(regions):
        if cv2.pointPolygonTest(numpy.array(region), point, False) >= 0: 
            return index
    return False



"""hekaiyue 何恺悦 2024-01-29"""
from . import b64
from tqdm import tqdm # python3 -m pip install tqdm -i https://pypi.tuna.tsinghua.edu.cn/simple
import cv2
import os
import requests
from pathlib import Path
import xml.etree.ElementTree as ET

def init(base_url, weight_path, config_path):
    # 查看模型是否已经被加载
    response = requests.post(url=f"{base_url}/status", json={})
    if "hky3535" in response.json()["response"]["grounding_dino"]: return {}
    # 如果没有被加载则上传模型并加载
    data = {
        "engine_name": "grounding_dino", 
        "engine_task": "hky3535", 
        "arguments": {
            "weight": {"name": Path(weight_path).name, "b64": b64.b_b64(open(weight_path, "rb").read())},
            "config": {"name": Path(config_path).name, "b64": b64.b_b64(open(config_path, "rb").read())}, 
            "device": "gpu"
        }
    }
    response = requests.post(url=f"{base_url}/load", json=data)
    return response.json()

def run(base_url, storage_path, classes):
    def infer(frame, classes):
        data = {
            "engine_name": "grounding_dino", 
            "engine_task": "hky3535", 
            "arguments": {
                "frame": {"b64": b64.cv_b64(frame)}, 
                "conf": 0.25, "iou": 0.7, 
                "text_prompt": classes, "text_threshold": 0.25
            }
        }
        response = requests.post(url=f"{base_url}/infer", json=data)
        return response.json()["response"]
    
    def save_xml(storage_path, file_stem, width, height, results, classes):
        # 递归转换：dict --> xml
        def dict2xml(element, data):
            for key, value in data.items():
                if isinstance(value, dict):
                    sub_element = ET.SubElement(element, key)
                    dict2xml(sub_element, value)
                elif isinstance(value, list):
                    for _item in value:
                        sub_element = ET.SubElement(element, "object")
                        dict2xml(sub_element, _item)
                else:
                    sub_element = ET.SubElement(element, key)
                    sub_element.text = str(value)

        # 整理出识别结果对应的基础dict结构
        results = {
            "filename": f"{file_stem}.jpg", 
            "size": {"width": width, "height": height, "depth": 3}, 
            "segmented": 0, 
            "object": [{
                "name": classes[cls], 
                "pose": "Unspecified", 
                "truncated": 0, 
                "difficult": 0, 
                "bndbox": {"xmin": x0, "ymin": y0, "xmax": x1, "ymax": y1,}
            } for x0, y0, x1, y1, conf, cls in results]
        }
        
        # 新建一个xml结构 并写入dict转换出来的xml并保存至本地
        root = ET.Element("annotation")
        dict2xml(root, results)
        ET.ElementTree(root).write(f"{storage_path}/{file_stem}.xml", encoding="utf-8", xml_declaration=True)
    
    # 仅接受jpg+xml的格式
    files_name = os.listdir(storage_path)
    files_stem = list(set([Path(file_name).stem for file_name in files_name]))
    
    # 主循环：逐个遍历图像
    for file_stem in tqdm(files_stem, total=len(files_stem)):
        # 如果对应xml已存在则不进行预标注
        if f"{file_stem}.xml" in files_name: continue
        # 读取一张图像
        frame = cv2.imread(f"{storage_path}/{file_stem}.jpg")
        width, height = frame.shape[:2]
        # 推理得到目标检测结果
        results = infer(frame=frame, classes=classes)
        # 整理结果并存入xml中
        save_xml(
            storage_path=storage_path, 
            file_stem=file_stem, 
            width=width, 
            height=height, 
            results=results, 
            classes=classes
        )



"""hekaiyue 何恺悦 2024-01-26"""
from os.path import dirname, abspath
from PIL import Image, ImageDraw, ImageFont
import cv2
import numpy

def draw_zh(frame, text="默认文字", position=(0, 0), color=(255, 255, 255), font=f"{dirname(abspath(__file__))}/data/MicrosoftYaHei-01.ttf", size=10):
    _frame = Image.fromarray(cv2.cvtColor(frame, cv2.COLOR_BGR2RGB))
    _draw = ImageDraw.Draw(_frame)
    _draw.text((position[0], position[1]), text=text, fill=color, font=ImageFont.truetype(font, size))
    frame = cv2.cvtColor(numpy.array(_frame), cv2.COLOR_RGB2BGR)
    return frame



import numpy
import random

def blank_frame(width, height):
    frame = numpy.zeros((height, width, 3), dtype=numpy.uint8)
    return frame

def moving_rectangles():
    return

def moving_points():
    return

def blinking_rectangles():
    return

def blinking_points(regions, hot_spots=False):
    return
